# @summary A short summary of the purpose of this class
#
# A description of what this class does
#
# @example
#   include cfssl
class cfssl (
  Hash $rootca_manifest = {},
  Stdlib::HTTPSUrl $downloadurl = 'https://github.com/cloudflare/cfssl/releases/download',
  String[1] $version = '1.6.3',
  String[1] $downloadchecksum = '824ff707e54611e8911210a5e8e1afbf745bba1dab55babdeb1a39d85b2f8c10',
  Enum['md5', 'sha1', 'sha2','sha256', 'sha384', 'sha512'] $checksum_type = 'sha256',
  String[1] $sysuser = 'cfssl',
  String[1] $sysgroup = 'cfssl',
  Stdlib::IP::Address $binding_ip = '127.0.0.1',
  Stdlib::Port $port = 8080,
  Enum['0','1','2','3','4'] $log_level = '1',
  Stdlib::Absolutepath $logdir = '/var/log/cfssl',
  String[1] $dbname = 'db_cfssl',
  String[1] $dbuser = 'u_cfssl',
  String[1] $dbpassword = 'changeme',
  Stdlib::Absolutepath $confdir = '/etc/cfssl',
  Stdlib::Absolutepath $binpath = '/usr/local/bin',
  Cfssl::Serveconfig $serve_config = { signing => { 'default' => { expiry => '1h', usages => ['client auth'] } } },
  String[1] $serve_ca = 'ROOT_ca',
) {
  include cfssl::goose
  include postgresql::server

  $_binaries = ['cfssljson','cfssl-certinfo']
  $_systemd_unit_file = '/etc/systemd/system/cfssl.service'
  $_goose_cfssldbmigrate_path = "/home/${sysuser}/goose-cfssldbmigrate"
  $_serve_config_json = 'serve-config.json'
  $_db_config_json = 'db-config.json'

  group { $sysgroup :
    ensure => present,
  }
  -> user { $sysuser :
    ensure     => present,
    managehome => true,
    shell      => '/bin/bash',
    gid        => $sysgroup,
  }

  archive::download { "${binpath}/cfssl" :
    ensure        => present,
    url           => "${downloadurl}/v${version}/cfssl_${version}_linux_amd64",
    checksum      => true,
    digest_type   => $checksum_type,
    digest_string => $downloadchecksum,
  }
  -> file { "${binpath}/cfssl" :
    ensure  => file,
    mode    => '0700',
    owner   => $sysuser,
    group   => $sysgroup,
    require => User[$sysuser],
  }

  $_binaries.each | String $_bin | {
    $_archiveurn = "v${version}/${_bin}_${version}_linux_amd64"

    archive::download { "${binpath}/${_bin}" :
      ensure    => present,
      url       => "${downloadurl}/${_archiveurn}",
      subscribe => Archive::Download["${binpath}/cfssl"],
    }
    ~> file { "${binpath}/${_bin}" :
      ensure  => file,
      mode    => '0700',
      owner   => $sysuser,
      group   => $sysgroup,
      require => User[$sysuser],
    }
  }

  file { [$confdir, "${confdir}/ca", $logdir]:
    ensure  => directory,
    mode    => '0700',
    owner   => $sysuser,
    group   => $sysgroup,
    require => User[$sysuser],
  }

  postgresql::server::db { $dbname:
    user     => $dbuser,
    password => postgresql::postgresql_password($dbuser, $dbpassword),
  }

  file { "${confdir}/${_db_config_json}":
    ensure  => file,
    mode    => '0600',
    owner   => $sysuser,
    group   => $sysgroup,
    content => epp('cfssl/db-config.json.epp'),
  }
  file { "${confdir}/${_serve_config_json}":
    ensure  => file,
    mode    => '0600',
    owner   => $sysuser,
    group   => $sysgroup,
    content => to_json_pretty($serve_config),
  }

  vcsrepo { $_goose_cfssldbmigrate_path:
    ensure   => present,
    provider => git,
    source   => 'https://github.com/cloudflare/cfssl.git',
    revision => "v${version}",
    user     => $sysuser,
  }
  -> file { "${_goose_cfssldbmigrate_path}/certdb/pg/dbconf.yml":
    ensure  => file,
    mode    => '0600',
    owner   => $sysuser,
    group   => $sysgroup,
    content => epp('cfssl/goose-dbconf.yml.epp'),
  }

  exec { 'goose pg up':
    command     => "/home/${sysuser}/go/bin/goose --env puppetmigrate -path ${_goose_cfssldbmigrate_path}/certdb/pg up",
    user        => $sysuser,
    environment => ["HOME=/home/${sysuser}/"],
    require     => [
      User[$sysuser],
      Vcsrepo[$_goose_cfssldbmigrate_path],
      Postgresql::Server::Db[$dbname],
    ],
    subscribe   => Vcsrepo[$_goose_cfssldbmigrate_path],
    refreshonly => true,
  }

  class { 'cfssl::ca::root':
    *       => $rootca_manifest,
    require => [
      File[$cfssl::confdir],
      File["${cfssl::confdir}/ca"],
      Archive::Download["${binpath}/cfssl"],
      Archive::Download["${binpath}/cfssljson"],
    ],
  }

  file { $_systemd_unit_file:
    ensure  => file,
    mode    => '0644',
    owner   => 0,
    group   => 0,
    content => epp('cfssl/cfssl.service.epp'),
  }
  ~> service { 'cfssl':
    ensure    => 'running',
    enable    => true,
    require   => [
      Archive::Download["${binpath}/cfssl"],
      Postgresql::Server::Db[$dbname],
      Exec['goose pg up'],
      File["${confdir}/${_serve_config_json}"],
      File["${confdir}/${_db_config_json}"],
      Class['cfssl::ca::root'],
    ],
    subscribe => Archive::Download["${binpath}/cfssl"],
    provider  => 'systemd',
  }
}
